@@ -105,7 +105,7 @@ http {
     }
 
     server {
-        listen {* prometheus_server_addr *};
+        listen {* prometheus_server_addr *} reuseport;
 
         access_log off;
 
@@ -578,7 +578,7 @@ http {
 
     {% if enabled_plugins["prometheus"] and prometheus_server_addr then %}
     server {
-        listen {* prometheus_server_addr *};
+        listen {* prometheus_server_addr *} reuseport;
 
         access_log off;
 
