@@ -94,7 +94,7 @@ do
             count = consumers_count_for_lrucache
         })
 
-local function construct_consumer_data(val, plugin_config)
+local function construct_consumer_data(val, name, plugin_config)
     -- if the val is a Consumer, clone it to the local consumer;
     -- if the val is a Credential, to get the Consumer by consumer_name and then clone
     -- it to the local consumer.
@@ -103,20 +103,28 @@ local function construct_consumer_data(val, plugin_config)
         local consumer_name = get_consumer_name_from_credential_etcd_key(val.key)
         local the_consumer = consumers:get(consumer_name)
         if the_consumer and the_consumer.value then
-            consumer = core.table.clone(the_consumer.value)
-            consumer.modifiedIndex = the_consumer.modifiedIndex
-            consumer.credential_id = get_credential_id_from_etcd_key(val.key)
+            consumer = consumers_id_lrucache(val.value.id .. name, val.modifiedIndex..
+                                                the_consumer.modifiedIndex,
+                function (val, the_consumer)
+                    consumer = core.table.clone(the_consumer.value)
+                    consumer.modifiedIndex = the_consumer.modifiedIndex
+                    consumer.credential_id = get_credential_id_from_etcd_key(val.key)
+                    return consumer
+                end, val, the_consumer)
         else
             -- Normally wouldn't get here:
             -- it should belong to a consumer for any credential.
-            core.log.error("failed to get the consumer for the credential,",
+            return nil, "failed to get the consumer for the credential,",
                 " a wild credential has appeared!",
-                " credential key: ", val.key, ", consumer name: ", consumer_name)
-            return nil, "failed to get the consumer for the credential"
+                " credential key: ", val.key, ", consumer name: ", consumer_name
         end
     else
-        consumer = core.table.clone(val.value)
-        consumer.modifiedIndex = val.modifiedIndex
+        consumer = consumers_id_lrucache(val.value.id .. name, val.modifiedIndex,
+            function (val)
+                consumer = core.table.clone(val.value)
+                consumer.modifiedIndex = val.modifiedIndex
+                return consumer
+            end, val)
     end
 
     -- if the consumer has labels, set the field custom_id to it.
@@ -160,16 +168,16 @@ function plugin_consumer()
                     }
                 end
 
-                local consumer = consumers_id_lrucache(val.value.id .. name,
-                        val.modifiedIndex, construct_consumer_data, val, config)
-                if consumer == nil then
+                local consumer, err = construct_consumer_data(val, name, config)
+                if not consumer then
+                    core.log.error("failed to construct consumer data for plugin ",
+                                   name, ": ", err)
                     goto CONTINUE
                 end
 
                 plugins[name].len = plugins[name].len + 1
                 core.table.insert(plugins[name].nodes, plugins[name].len,
                                     consumer)
-                core.log.info("consumer:", core.json.delay_encode(consumer))
             end
         end
 
@@ -239,7 +247,6 @@ function create_consume_cache(consumers_conf, key_attr)
     local consumer_names = {}
 
     for _, consumer in ipairs(consumers_conf.nodes) do
-        core.log.info("consumer node: ", core.json.delay_encode(consumer))
         local new_consumer = consumer_lrucache(consumer, nil,
                                 fill_consumer_secret, consumer)
         consumer_names[new_consumer.auth_conf[key_attr]] = new_consumer
