@@ -22,6 +22,8 @@
 local require           = require
 local fetch_local_conf  = require("apisix.core.config_local").local_conf
 local array_mt          = require("apisix.core.json").array_mt
+local log               = require("apisix.core.log")
+local try_read_attr     = require("apisix.core.table").try_read_attr
 local v3_adapter        = require("apisix.admin.v3_adapter")
 local etcd              = require("resty.etcd")
 local clone_tab         = require("table.clone")
@@ -37,6 +39,81 @@ local ngx_get_phase     = ngx.get_phase
 local _M = {}
 
 
+local NOT_ALLOW_WRITE_ETCD_WARN = 'Data plane role should not write to etcd. ' ..
+    'This operation will be deprecated in future releases.'
+
+local function is_data_plane()
+    local local_conf, err = fetch_local_conf()
+    if not local_conf then
+        return nil, err
+    end
+
+    local role = try_read_attr(local_conf, "deployment", "role")
+    if role == "data_plane" then
+      return true
+    end
+
+    return false
+end
+
+
+
+local function disable_write_if_data_plane()
+    local data_plane, err = is_data_plane()
+    if err then
+        log.error("failed to check data plane role: ", err)
+        return true, err
+    end
+
+    if data_plane then
+        -- current only warn, will be return false in future releases
+        -- to block etcd write
+        log.warn(NOT_ALLOW_WRITE_ETCD_WARN)
+        return false
+    end
+
+    return false, nil
+end
+
+
+local function wrap_etcd_client(etcd_cli)
+    -- note: methods txn can read and write, don't use txn to write when data plane role
+    local methods_to_wrap = {
+        "set",
+        "setnx",
+        "setx",
+        "delete",
+        "rmdir",
+        "grant",
+        "revoke",
+        "keepalive"
+    }
+
+    local original_methods = {}
+    for _, method in ipairs(methods_to_wrap) do
+        if not etcd_cli[method] then
+            log.error("method ", method, " not found in etcd client")
+            return nil, "method " .. method .. " not found in etcd client"
+        end
+
+        original_methods[method] = etcd_cli[method]
+    end
+
+    for _, method in ipairs(methods_to_wrap) do
+        etcd_cli[method] = function(self, ...)
+            local disable, err = disable_write_if_data_plane()
+            if disable then
+                return nil, err
+            end
+
+            return original_methods[method](self, ...)
+        end
+    end
+
+    return etcd_cli
+end
+
+
 local function _new(etcd_conf)
     local prefix = etcd_conf.prefix
     etcd_conf.http_host = etcd_conf.host
@@ -67,6 +144,8 @@ local function _new(etcd_conf)
         return nil, nil, err
     end
 
+    etcd_cli = wrap_etcd_client(etcd_cli)
+
     return etcd_cli, prefix
 end
 
@@ -337,6 +416,12 @@ end
 
 
 local function set(key, value, ttl)
+    local disable, err = disable_write_if_data_plane()
+    if disable then
+        return nil, err
+    end
+
+
     local etcd_cli, prefix, err = get_etcd_cli()
     if not etcd_cli then
         return nil, err
@@ -386,6 +471,11 @@ _M.set = set
 
 
 function _M.atomic_set(key, value, ttl, mod_revision)
+    local disable, err = disable_write_if_data_plane()
+    if disable then
+        return nil, err
+    end
+
     local etcd_cli, prefix, err = get_etcd_cli()
     if not etcd_cli then
         return nil, err
@@ -443,7 +533,13 @@ function _M.atomic_set(key, value, ttl, mod_revision)
 end
 
 
+
 function _M.push(key, value, ttl)
+    local disable, err = disable_write_if_data_plane()
+    if disable then
+        return nil, err
+    end
+
     local etcd_cli, _, err = get_etcd_cli()
     if not etcd_cli then
         return nil, err
@@ -476,6 +572,11 @@ end
 
 
 function _M.delete(key)
+    local disable, err = disable_write_if_data_plane()
+    if disable then
+        return nil, err
+    end
+
     local etcd_cli, prefix, err = get_etcd_cli()
     if not etcd_cli then
         return nil, err
@@ -502,6 +603,11 @@ function _M.delete(key)
 end
 
 function _M.rmdir(key, opts)
+    local disable, err = disable_write_if_data_plane()
+    if disable then
+        return nil, err
+    end
+
     local etcd_cli, prefix, err = get_etcd_cli()
     if not etcd_cli then
         return nil, err
@@ -548,6 +654,11 @@ end
 
 
 function _M.keepalive(id)
+    local disable, err = disable_write_if_data_plane()
+    if disable then
+        return nil, err
+    end
+
     local etcd_cli, _, err = get_etcd_cli()
     if not etcd_cli then
         return nil, err
