@@ -19,15 +19,32 @@
 --
 -- @module core.resolver
 
-local json = require("apisix.core.json")
-local log = require("apisix.core.log")
-local utils = require("apisix.core.utils")
+local json           = require("apisix.core.json")
+local log            = require("apisix.core.log")
+local utils          = require("apisix.core.utils")
+local dns_utils      = require("resty.dns.utils")
+local config_local   = require("apisix.core.config_local")
+
+
+local HOSTS_IP_MATCH_CACHE = {}
 
 
 local _M = {}
 
 
+local function init_hosts_ip()
+    local hosts, err = dns_utils.parseHosts()
+    if not hosts then
+        return hosts, err
+    end
+    HOSTS_IP_MATCH_CACHE = hosts
+end
+
+
 function _M.init_resolver(args)
+    --  initialize /etc/hosts
+    init_hosts_ip()
+
     local dns_resolver = args and args["dns_resolver"]
     utils.set_resolver(dns_resolver)
     log.info("dns resolver ", json.delay_encode(dns_resolver, true))
@@ -42,6 +59,22 @@ end
 -- @usage
 -- local ip, err = core.resolver.parse_domain("apache.org") -- "198.18.10.114"
 function _M.parse_domain(host)
+    local rev = HOSTS_IP_MATCH_CACHE[host]
+    local enable_ipv6 = config_local.local_conf().apisix.enable_ipv6
+    if rev then
+        -- use ipv4 in high priority
+        local ip = rev["ipv4"]
+        if enable_ipv6 and not ip then
+            ip = rev["ipv6"]
+        end
+        if ip then
+            -- meet test case
+            log.info("dns resolve ", host, ", result: ", json.delay_encode(ip))
+            log.info("dns resolver domain: ", host, " to ", ip)
+            return ip
+        end
+    end
+
     local ip_info, err = utils.dns_parse(host)
     if not ip_info then
         log.error("failed to parse domain: ", host, ", error: ",err)
