@@ -23,15 +23,17 @@ local mt = {
 local CONTENT_TYPE_JSON = "application/json"
 
 local core = require("apisix.core")
+local plugin = require("apisix.plugin")
 local http = require("resty.http")
 local url  = require("socket.url")
-local ngx_re = require("ngx.re")
-
-local ngx_print = ngx.print
-local ngx_flush = ngx.flush
+local sse  = require("apisix.plugins.ai-drivers.sse")
+local ngx  = ngx
+local ngx_now = ngx.now
 
+local table = table
 local pairs = pairs
 local type  = type
+local math  = math
 local ipairs = ipairs
 local setmetatable = setmetatable
 
@@ -45,6 +47,7 @@ function _M.new(opts)
         host = opts.host,
         port = opts.port,
         path = opts.path,
+        remove_model = opts.options and opts.options.remove_model
     }
     return setmetatable(self, mt)
 end
@@ -84,8 +87,11 @@ local function read_response(ctx, res)
     core.response.set_header("Content-Type", content_type)
 
     if content_type and core.string.find(content_type, "text/event-stream") then
+        local contents = {}
         while true do
             local chunk, err = body_reader() -- will read chunk by chunk
+            ctx.var.apisix_upstream_response_time = math.floor((ngx_now() -
+                                             ctx.llm_request_start_time) * 1000)
             if err then
                 core.log.warn("failed to read response chunk: ", err)
                 return handle_error(err)
@@ -94,50 +100,56 @@ local function read_response(ctx, res)
                 return
             end
 
-            ngx_print(chunk)
-            ngx_flush(true)
-
-            local events, err = ngx_re.split(chunk, "\n")
-            if err then
-                core.log.warn("failed to split response chunk [", chunk, "] to events: ", err)
-                goto CONTINUE
+            if ctx.var.llm_time_to_first_token == "0" then
+                ctx.var.llm_time_to_first_token = math.floor(
+                                                (ngx_now() - ctx.llm_request_start_time) * 1000)
             end
 
+            local events = sse.decode(chunk)
+            ctx.llm_response_contents_in_chunk = {}
             for _, event in ipairs(events) do
-                if not core.string.find(event, "data:") or core.string.find(event, "[DONE]") then
-                    goto CONTINUE
+                if event.type == "message" then
+                    local data, err = core.json.decode(event.data)
+                    if not data then
+                        core.log.warn("failed to decode SSE data: ", err)
+                        goto CONTINUE
+                    end
+
+                    if data and type(data.choices) == "table" and #data.choices > 0 then
+                        for _, choice in ipairs(data.choices) do
+                            if type(choice) == "table"
+                                    and type(choice.delta) == "table"
+                                    and type(choice.delta.content) == "string" then
+                                core.table.insert(contents, choice.delta.content)
+                                core.table.insert(ctx.llm_response_contents_in_chunk,
+                                                        choice.delta.content)
+                            end
+                        end
+                    end
+
+
+                    -- usage field is null for non-last events, null is parsed as userdata type
+                    if data and type(data.usage) == "table" then
+                        core.log.info("got token usage from ai service: ",
+                                            core.json.delay_encode(data.usage))
+                        ctx.llm_raw_usage = data.usage
+                        ctx.ai_token_usage = {
+                            prompt_tokens = data.usage.prompt_tokens or 0,
+                            completion_tokens = data.usage.completion_tokens or 0,
+                            total_tokens = data.usage.total_tokens or 0,
+                        }
+                        ctx.var.llm_prompt_tokens = ctx.ai_token_usage.prompt_tokens
+                        ctx.var.llm_completion_tokens = ctx.ai_token_usage.completion_tokens
+                        ctx.var.llm_response_text = table.concat(contents, "")
+                    end
+                elseif event.type == "done" then
+                    ctx.var.llm_request_done = true
                 end
 
-                local parts, err = ngx_re.split(event, ":", nil, nil, 2)
-                if err then
-                    core.log.warn("failed to split data event [", event,  "] to parts: ", err)
-                    goto CONTINUE
-                end
-
-                if #parts ~= 2 then
-                    core.log.warn("malformed data event: ", event)
-                    goto CONTINUE
-                end
-
-                local data, err = core.json.decode(parts[2])
-                if err then
-                    core.log.warn("failed to decode data event [", parts[2], "] to json: ", err)
-                    goto CONTINUE
-                end
-
-                -- usage field is null for non-last events, null is parsed as userdata type
-                if data and data.usage and type(data.usage) ~= "userdata" then
-                    core.log.info("got token usage from ai service: ",
-                                        core.json.delay_encode(data.usage))
-                    ctx.ai_token_usage = {
-                        prompt_tokens = data.usage.prompt_tokens or 0,
-                        completion_tokens = data.usage.completion_tokens or 0,
-                        total_tokens = data.usage.total_tokens or 0,
-                    }
-                end
+                ::CONTINUE::
             end
 
-            ::CONTINUE::
+            plugin.lua_response_filter(ctx, res.headers, chunk)
         end
     end
 
@@ -146,19 +158,38 @@ local function read_response(ctx, res)
         core.log.warn("failed to read response body: ", err)
         return handle_error(err)
     end
+    ngx.status = res.status
+    ctx.var.llm_time_to_first_token = math.floor((ngx_now() - ctx.llm_request_start_time) * 1000)
+    ctx.var.apisix_upstream_response_time = ctx.var.llm_time_to_first_token
     local res_body, err = core.json.decode(raw_res_body)
     if err then
         core.log.warn("invalid response body from ai service: ", raw_res_body, " err: ", err,
             ", it will cause token usage not available")
     else
         core.log.info("got token usage from ai service: ", core.json.delay_encode(res_body.usage))
-        ctx.ai_token_usage = {
-            prompt_tokens = res_body.usage and res_body.usage.prompt_tokens or 0,
-            completion_tokens = res_body.usage and res_body.usage.completion_tokens or 0,
-            total_tokens = res_body.usage and res_body.usage.total_tokens or 0,
-        }
+        ctx.ai_token_usage = {}
+        if type(res_body.usage) == "table" then
+            ctx.llm_raw_usage = res_body.usage
+            ctx.ai_token_usage.prompt_tokens = res_body.usage.prompt_tokens or 0
+            ctx.ai_token_usage.completion_tokens = res_body.usage.completion_tokens or 0
+            ctx.ai_token_usage.total_tokens = res_body.usage.total_tokens or 0
+        end
+        ctx.var.llm_prompt_tokens = ctx.ai_token_usage.prompt_tokens or 0
+        ctx.var.llm_completion_tokens = ctx.ai_token_usage.completion_tokens or 0
+        if type(res_body.choices) == "table" and #res_body.choices > 0 then
+            local contents = {}
+            for _, choice in ipairs(res_body.choices) do
+                if type(choice) == "table"
+                        and type(choice.message) == "table"
+                        and type(choice.message.content) == "string" then
+                    core.table.insert(contents, choice.message.content)
+                end
+            end
+            local content_to_check = table.concat(contents, " ")
+            ctx.var.llm_response_text = content_to_check
+        end
     end
-    return res.status, raw_res_body
+    plugin.lua_response_filter(ctx, res.headers, raw_res_body)
 end
 
 
@@ -170,7 +201,7 @@ function _M.request(self, ctx, conf, request_table, extra_opts)
     end
     httpc:set_timeout(conf.timeout)
 
-    local endpoint = extra_opts.endpoint
+    local endpoint = extra_opts and extra_opts.endpoint
     local parsed_url
     if endpoint then
         parsed_url = url.parse(endpoint)
@@ -225,7 +256,9 @@ function _M.request(self, ctx, conf, request_table, extra_opts)
             request_table[opt] = val
         end
     end
-
+    if self.remove_model then
+        request_table.model = nil
+    end
     local req_json, err = core.json.encode(request_table)
     if not req_json then
         return nil, err
@@ -239,6 +272,11 @@ function _M.request(self, ctx, conf, request_table, extra_opts)
         return handle_error(err)
     end
 
+    -- handling this error separately is needed for retries
+    if res.status == 429 or (res.status >= 500 and res.status < 600 )then
+        return res.status
+    end
+
     local code, body = read_response(ctx, res)
 
     if conf.keepalive then
