@@ -187,6 +187,11 @@ function _M.check_schema(conf, schema_type)
     if schema_type == core.schema.TYPE_METADATA then
         return core.schema.check(metadata_schema, conf)
     end
+
+    local check = {"skywalking.endpoint_addr", "clickhouse.endpoint_addr"}
+    core.utils.check_https(check, conf, plugin_name)
+    core.utils.check_tls_bool({"tcp.tls"}, conf, plugin_name)
+
     return core.schema.check(schema, conf)
 end
 
