@@ -15,13 +15,13 @@
 -- limitations under the License.
 --
 local base_prometheus = require("prometheus")
+local tonumber        = tonumber
 local core      = require("apisix.core")
 local plugin    = require("apisix.plugin")
 local control   = require("apisix.control.v1")
 local ipairs    = ipairs
 local pairs     = pairs
 local ngx       = ngx
-local re_gmatch = ngx.re.gmatch
 local ffi       = require("ffi")
 local C         = ffi.C
 local pcall = pcall
@@ -45,23 +45,33 @@ local latency_details = require("apisix.utils.log-util").latency_details_in_ms
 local xrpc = require("apisix.stream.xrpc")
 local unpack = unpack
 local next = next
+local process = require("ngx.process")
+local tonumber = tonumber
+local shdict_prometheus_cache = ngx.shared["prometheus-cache"]
+local ngx_timer_every = ngx.timer.every
 
-
-local ngx_capture
-if ngx.config.subsystem == "http" then
-    ngx_capture = ngx.location.capture
+if not shdict_prometheus_cache then
+    error("lua_shared_dict \"prometheus-cache\" not configured")
 end
 
-
 local plugin_name = "prometheus"
 local default_export_uri = "/apisix/prometheus/metrics"
 -- Default set of latency buckets, 1ms to 60s:
 local DEFAULT_BUCKETS = {1, 2, 5, 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 30000, 60000}
+-- Default refresh interval
+local DEFAULT_REFRESH_INTERVAL = 15
+
+local CACHED_METRICS_KEY = "cached_metrics_text"
 
 local metrics = {}
 
 local inner_tab_arr = {}
 
+local exporter_timer_running = false
+
+local exporter_timer_created = false
+
+
 local function gen_arr(...)
     clear_tab(inner_tab_arr)
     for i = 1, select('#', ...) do
@@ -110,6 +120,15 @@ end
 
 
 function _M.http_init(prometheus_enabled_in_stream)
+    -- FIXME:
+    -- Now the HTTP subsystem loads the stream plugin unintentionally, which shouldn't happen.
+    -- But this part did not show any issues during testing,
+    -- it's just to prevent the same problem from happening again.
+    if ngx.config.subsystem ~= "http" then
+        core.log.warn("prometheus plugin http_init should not be called in stream subsystem")
+        return
+    end
+
     -- todo: support hot reload, we may need to update the lua-prometheus
     -- library
     if ngx.get_phase() ~= "init" and ngx.get_phase() ~= "init_worker"  then
@@ -120,7 +139,6 @@ function _M.http_init(prometheus_enabled_in_stream)
     end
 
     clear_tab(metrics)
-
     -- Newly added metrics should follow the naming best practices described in
     -- https://prometheus.io/docs/practices/naming/#metric-names
     -- For example,
@@ -144,6 +162,13 @@ function _M.http_init(prometheus_enabled_in_stream)
                                    "bandwidth", "expire")
     local upstream_status_exptime = core.table.try_read_attr(attr, "metrics",
                                    "upstream_status", "expire")
+    local llm_latency_exptime = core.table.try_read_attr(attr, "metrics", "llm_latency", "expire")
+    local llm_prompt_tokens_exptime = core.table.try_read_attr(attr, "metrics",
+                                                            "llm_prompt_tokens", "expire")
+    local llm_completion_tokens_exptime = core.table.try_read_attr(attr, "metrics",
+                                                            "llm_completion_tokens", "expire")
+    local llm_active_connections_exptime = core.table.try_read_attr(attr, "metrics",
+                                                            "llm_active_connections", "expire")
 
     prometheus = base_prometheus.init("prometheus-metrics", metric_prefix)
 
@@ -186,6 +211,7 @@ function _M.http_init(prometheus_enabled_in_stream)
     metrics.status = prometheus:counter("http_status",
             "HTTP status codes per service in APISIX",
             {"code", "route", "matched_uri", "matched_host", "service", "consumer", "node",
+            "request_type", "request_llm_model", "llm_model",
             unpack(extra_labels("http_status"))},
             status_metrics_exptime)
 
@@ -196,14 +222,52 @@ function _M.http_init(prometheus_enabled_in_stream)
 
     metrics.latency = prometheus:histogram("http_latency",
         "HTTP request latency in milliseconds per service in APISIX",
-        {"type", "route", "service", "consumer", "node", unpack(extra_labels("http_latency"))},
+        {"type", "route", "service", "consumer", "node",
+        "request_type", "request_llm_model", "llm_model",
+        unpack(extra_labels("http_latency"))},
         buckets, latency_metrics_exptime)
 
     metrics.bandwidth = prometheus:counter("bandwidth",
             "Total bandwidth in bytes consumed per service in APISIX",
-            {"type", "route", "service", "consumer", "node", unpack(extra_labels("bandwidth"))},
+            {"type", "route", "service", "consumer", "node",
+            "request_type", "request_llm_model", "llm_model",
+            unpack(extra_labels("bandwidth"))},
             bandwidth_metrics_exptime)
 
+    local llm_latency_buckets = DEFAULT_BUCKETS
+    if attr and attr.llm_latency_buckets then
+        llm_latency_buckets = attr.llm_latency_buckets
+    end
+    metrics.llm_latency = prometheus:histogram("llm_latency",
+        "LLM request latency in milliseconds",
+        {"route_id", "service_id", "consumer", "node",
+        "request_type", "request_llm_model", "llm_model",
+        unpack(extra_labels("llm_latency"))},
+        llm_latency_buckets,
+        llm_latency_exptime)
+
+    metrics.llm_prompt_tokens = prometheus:counter("llm_prompt_tokens",
+            "LLM service consumed prompt tokens",
+            {"route_id", "service_id", "consumer", "node",
+            "request_type", "request_llm_model", "llm_model",
+            unpack(extra_labels("llm_prompt_tokens"))},
+            llm_prompt_tokens_exptime)
+
+    metrics.llm_completion_tokens = prometheus:counter("llm_completion_tokens",
+            "LLM service consumed completion tokens",
+            {"route_id", "service_id", "consumer", "node",
+            "request_type", "request_llm_model", "llm_model",
+            unpack(extra_labels("llm_completion_tokens"))},
+            llm_completion_tokens_exptime)
+
+    metrics.llm_active_connections = prometheus:gauge("llm_active_connections",
+            "Number of active connections to LLM service",
+            {"route", "route_id", "matched_uri", "matched_host",
+            "service", "service_id", "consumer", "node",
+            "request_type", "request_llm_model", "llm_model",
+            unpack(extra_labels("llm_active_connections"))},
+            llm_active_connections_exptime)
+
     if prometheus_enabled_in_stream then
         init_stream_metrics()
     end
@@ -211,6 +275,15 @@ end
 
 
 function _M.stream_init()
+    -- FIXME:
+    -- Now the HTTP subsystem loads the stream plugin unintentionally, which shouldn't happen.
+    -- It breaks the initialization logic of the plugin,
+    -- here it is temporarily fixed using a workaround.
+    if ngx.config.subsystem ~= "stream" then
+        core.log.warn("prometheus plugin stream_init should not be called in http subsystem")
+        return
+    end
+
     if ngx.get_phase() ~= "init" and ngx.get_phase() ~= "init_worker"  then
         return
     end
@@ -265,6 +338,7 @@ function _M.http_log(conf, ctx)
     metrics.status:inc(1,
         gen_arr(vars.status, route_id, matched_uri, matched_host,
                 service_id, consumer_name, balancer_ip,
+                vars.request_type, vars.request_llm_model, vars.llm_model,
                 unpack(extra_labels("http_status", ctx))))
 
     local latency, upstream_latency, apisix_latency = latency_details(ctx)
@@ -272,27 +346,52 @@ function _M.http_log(conf, ctx)
 
     metrics.latency:observe(latency,
         gen_arr("request", route_id, service_id, consumer_name, balancer_ip,
+        vars.request_type, vars.request_llm_model, vars.llm_model,
         unpack(latency_extra_label_values)))
 
     if upstream_latency then
         metrics.latency:observe(upstream_latency,
             gen_arr("upstream", route_id, service_id, consumer_name, balancer_ip,
+            vars.request_type, vars.request_llm_model, vars.llm_model,
             unpack(latency_extra_label_values)))
     end
 
     metrics.latency:observe(apisix_latency,
         gen_arr("apisix", route_id, service_id, consumer_name, balancer_ip,
+        vars.request_type, vars.request_llm_model, vars.llm_model,
         unpack(latency_extra_label_values)))
 
     local bandwidth_extra_label_values = extra_labels("bandwidth", ctx)
 
     metrics.bandwidth:inc(vars.request_length,
         gen_arr("ingress", route_id, service_id, consumer_name, balancer_ip,
+        vars.request_type, vars.request_llm_model, vars.llm_model,
         unpack(bandwidth_extra_label_values)))
 
     metrics.bandwidth:inc(vars.bytes_sent,
         gen_arr("egress", route_id, service_id, consumer_name, balancer_ip,
+        vars.request_type, vars.request_llm_model, vars.llm_model,
         unpack(bandwidth_extra_label_values)))
+
+    local llm_time_to_first_token = vars.llm_time_to_first_token
+    if llm_time_to_first_token ~= "" then
+        metrics.llm_latency:observe(tonumber(llm_time_to_first_token),
+            gen_arr(route_id, service_id, consumer_name, balancer_ip,
+            vars.request_type, vars.request_llm_model, vars.llm_model,
+            unpack(extra_labels("llm_latency", ctx))))
+    end
+    if vars.llm_prompt_tokens ~= "" then
+        metrics.llm_prompt_tokens:inc(tonumber(vars.llm_prompt_tokens),
+            gen_arr(route_id, service_id, consumer_name, balancer_ip,
+            vars.request_type, vars.request_llm_model, vars.llm_model,
+            unpack(extra_labels("llm_prompt_tokens", ctx))))
+    end
+    if vars.llm_completion_tokens ~= "" then
+        metrics.llm_completion_tokens:inc(tonumber(vars.llm_completion_tokens),
+            gen_arr(route_id, service_id, consumer_name, balancer_ip,
+            vars.request_type, vars.request_llm_model, vars.llm_model,
+            unpack(extra_labels("llm_completion_tokens", ctx))))
+    end
 end
 
 
@@ -310,39 +409,50 @@ function _M.stream_log(conf, ctx)
 end
 
 
-local ngx_status_items = {"active", "accepted", "handled", "total",
-                         "reading", "writing", "waiting"}
+-- FFI definitions for nginx connection status
+-- Based on https://github.com/nginx/nginx/blob/master/src/event/ngx_event.c#L61-L78
+ffi.cdef[[
+    typedef uint64_t ngx_atomic_uint_t;
+    extern ngx_atomic_uint_t  *ngx_stat_accepted;
+    extern ngx_atomic_uint_t  *ngx_stat_handled;
+    extern ngx_atomic_uint_t  *ngx_stat_requests;
+    extern ngx_atomic_uint_t  *ngx_stat_active;
+    extern ngx_atomic_uint_t  *ngx_stat_reading;
+    extern ngx_atomic_uint_t  *ngx_stat_writing;
+    extern ngx_atomic_uint_t  *ngx_stat_waiting;
+]]
+
 local label_values = {}
 
+-- Mapping of status names to FFI global variables and metrics
+local status_mapping = {
+    {name = "active", var = "ngx_stat_active"},
+    {name = "accepted", var = "ngx_stat_accepted"},
+    {name = "handled", var = "ngx_stat_handled"},
+    {name = "total", var = "ngx_stat_requests"},
+    {name = "reading", var = "ngx_stat_reading"},
+    {name = "writing", var = "ngx_stat_writing"},
+    {name = "waiting", var = "ngx_stat_waiting"},
+}
+
+-- Use FFI to get nginx status directly from global variables
 local function nginx_status()
-    local res = ngx_capture("/apisix/nginx_status")
-    if not res or res.status ~= 200 then
-        core.log.error("failed to fetch Nginx status")
-        return
-    end
+    for _, item in ipairs(status_mapping) do
+        local ok, value = pcall(function()
+            local stat_ptr = C[item.var]
+            return stat_ptr and tonumber(stat_ptr[0]) or 0
+        end)
 
-    -- Active connections: 2
-    -- server accepts handled requests
-    --   26 26 84
-    -- Reading: 0 Writing: 1 Waiting: 1
-
-    local iterator, err = re_gmatch(res.body, [[(\d+)]], "jmo")
-    if not iterator then
-        core.log.error("failed to re.gmatch Nginx status: ", err)
-        return
-    end
-
-    for _, name in ipairs(ngx_status_items) do
-        local val = iterator()
-        if not val then
-            break
+        if not ok then
+            core.log.error("failed to read ", item.name, " via FFI")
+            return
         end
 
-        if name == "total" then
-            metrics.requests:set(val[0])
+        if item.name == "total" then
+            metrics.requests:set(value)
         else
-            label_values[1] = name
-            metrics.connections:set(val[0], label_values)
+            label_values[1] = item.name
+            metrics.connections:set(value, label_values)
         end
     end
 end
@@ -438,13 +548,7 @@ local function shared_dict_status()
 end
 
 
-local function collect(ctx, stream_only)
-    if not prometheus or not metrics then
-        core.log.error("prometheus: plugin is not initialized, please make sure ",
-                     " 'prometheus_metrics' shared dict is present in nginx template")
-        return 500, {message = "An unexpected error occurred"}
-    end
-
+local function collect(yieldable)
     -- collect ngx.shared.DICT status
     shared_dict_status()
 
@@ -454,34 +558,37 @@ local function collect(ctx, stream_only)
     local config = core.config.new()
 
     -- config server status
-    local vars = ngx.var or {}
-    local hostname = vars.hostname or ""
+    local hostname = core.utils.gethostname() or ""
     local version = core.version.VERSION or ""
 
+    local local_conf = core.config.local_conf()
+    local stream_only = local_conf.apisix.proxy_mode == "stream"
     -- we can't get etcd index in metric server if only stream subsystem is enabled
     if config.type == "etcd" and not stream_only then
         -- etcd modify index
         etcd_modify_index()
 
-        local version, err = config:server_version()
-        if version then
-            metrics.etcd_reachable:set(1)
+        if yieldable then
+            local version, err = config:server_version()
+            if version then
+                metrics.etcd_reachable:set(1)
 
-        else
-            metrics.etcd_reachable:set(0)
-            core.log.error("prometheus: failed to reach config server while ",
-                           "processing metrics endpoint: ", err)
-        end
+            else
+                metrics.etcd_reachable:set(0)
+                core.log.error("prometheus: failed to reach config server while ",
+                            "processing metrics endpoint: ", err)
+            end
 
-        -- Because request any key from etcd will return the "X-Etcd-Index".
-        -- A non-existed key is preferred because it doesn't return too much data.
-        -- So use phantom key to get etcd index.
-        local res, _ = config:getkey("/phantomkey")
-        if res and res.headers then
-            clear_tab(key_values)
-            -- global max
-            key_values[1] = "x_etcd_index"
-            metrics.etcd_modify_indexes:set(res.headers["X-Etcd-Index"], key_values)
+            -- Because request any key from etcd will return the "X-Etcd-Index".
+            -- A non-existed key is preferred because it doesn't return too much data.
+            -- So use phantom key to get etcd index.
+            local res, _ = config:getkey("/phantomkey")
+            if res and res.headers then
+                clear_tab(key_values)
+                -- global max
+                key_values[1] = "x_etcd_index"
+                metrics.etcd_modify_indexes:set(res.headers["X-Etcd-Index"], key_values)
+            end
         end
     end
 
@@ -498,10 +605,104 @@ local function collect(ctx, stream_only)
         end
     end
 
+    return core.table.concat(prometheus:metric_data())
+end
+
+
+local function exporter_timer(premature, yieldable, cache_exptime)
+    if premature then
+        return
+    end
+
+    if not prometheus then
+        return
+    end
+
+    if exporter_timer_running then
+        core.log.warn("Previous calculation still running, skipping")
+        return
+    end
+
+    exporter_timer_running = true
+
+    local ok, res = pcall(collect, yieldable)
+    if not ok then
+        core.log.error("Failed to collect metrics: ", res)
+        exporter_timer_running = false
+        return
+    end
+
+    -- Clear the cached data after cache_exptime to prevent stale data in case of an error.
+    local _, err, forcible = shdict_prometheus_cache:set(CACHED_METRICS_KEY, res, cache_exptime)
+    if err then
+        core.log.error("Failed to save metrics to the `prometheus-cache` shared dict: ", err,
+                    ". The size of the value being attempted to be saved is: ", #res)
+    end
+
+    if forcible then
+        core.log.error("Shared dictionary used for prometheus cache " ..
+  "is full. REPORTED METRIC DATA MIGHT BE INCOMPLETE. Please increase the " ..
+  "size of the `prometheus-cache` shared dict or decrease metric cardinality.")
+    end
+
+    exporter_timer_running = false
+end
+
+
+local function init_exporter_timer()
+    if process.type() ~= "privileged agent" then
+        return
+    end
+
+    local refresh_interval = DEFAULT_REFRESH_INTERVAL
+    local attr = plugin.plugin_attr("prometheus")
+    if attr and attr.refresh_interval then
+        refresh_interval = attr.refresh_interval
+    end
+
+    local cache_exptime = refresh_interval * 2
+
+    exporter_timer(false, false, cache_exptime)
+
+    if exporter_timer_created then
+        core.log.info("Exporter timer already created, skipping")
+        return
+    end
+
+    -- When the APISIX configuration `refresh_interval` is updated,
+    -- the timer will not restart, and the new `refresh_interval` will not be applied.
+    -- APISIX needs to be restarted to apply the changes.
+    local ok, err = ngx_timer_every(refresh_interval, exporter_timer, true, cache_exptime)
+
+    if ok then
+        exporter_timer_created = true
+    else
+        core.log.error("Failed to start the exporter timer: ", err)
+    end
+end
+_M.init_exporter_timer = init_exporter_timer
+
+
+local function get_cached_metrics()
+    if not prometheus or not metrics then
+        core.log.error("prometheus: plugin is not initialized, please make sure ",
+                     " 'prometheus_metrics' shared dict is present in nginx template")
+        return 500, {message = "An unexpected error occurred"}
+    end
+
     core.response.set_header("content_type", "text/plain")
-    return 200, core.table.concat(prometheus:metric_data())
+    local cached_metrics_text, err = shdict_prometheus_cache:get(CACHED_METRICS_KEY)
+    if err then
+        core.log.error("Failed to retrieve cached metrics: err: ", err)
+        return 500, {message = "Failed to retrieve metrics. err: " .. err}
+    end
+    if not cached_metrics_text then
+        core.log.error("Failed to retrieve cached metrics: data is nil")
+        return 500, {message = "Failed to retrieve metrics: no data available"}
+    end
+
+    return 200, cached_metrics_text
 end
-_M.collect = collect
 
 
 local function get_api(called_by_api_router)
@@ -514,7 +715,7 @@ local function get_api(called_by_api_router)
     local api = {
         methods = {"GET"},
         uri = export_uri,
-        handler = collect
+        handler = get_cached_metrics
     }
 
     if not called_by_api_router then
@@ -530,16 +731,16 @@ end
 _M.get_api = get_api
 
 
-function _M.export_metrics(stream_only)
+function _M.export_metrics()
     if not prometheus then
-        core.response.exit(200, "{}")
+        return core.response.exit(200, "{}")
     end
     local api = get_api(false)
     local uri = ngx.var.uri
     local method = ngx.req.get_method()
 
     if uri == api.uri and method == api.methods[1] then
-        local code, body = api.handler(nil, stream_only)
+        local code, body = api.handler()
         if code or body then
             core.response.exit(code, body)
         end
@@ -553,6 +754,54 @@ function _M.metric_data()
     return prometheus:metric_data()
 end
 
+local function inc_llm_active_connections(ctx, value)
+
+    local vars = ctx.var
+
+    local route_id = ""
+    local route_name = ""
+    local balancer_ip = ctx.balancer_ip or ""
+    local service_id = ""
+    local service_name = ""
+    local consumer_name = ctx.consumer_name or ""
+
+    local matched_route = ctx.matched_route and ctx.matched_route.value
+    if matched_route then
+        route_id = matched_route.id
+        route_name = matched_route.name or ""
+        service_id = matched_route.service_id or ""
+        if service_id ~= "" then
+            local fetched_service = service_fetch(service_id)
+            service_name = fetched_service and fetched_service.value.name or ""
+        end
+    end
+
+    local matched_uri = ""
+    local matched_host = ""
+    if ctx.curr_req_matched then
+        matched_uri = ctx.curr_req_matched._path or ""
+        matched_host = ctx.curr_req_matched._host or ""
+    end
+
+    metrics.llm_active_connections:inc(
+        value,
+        gen_arr(route_name, route_id, matched_uri,
+            matched_host, service_name, service_id, consumer_name, balancer_ip,
+            vars.request_type, vars.request_llm_model, vars.llm_model,
+        unpack(extra_labels("llm_active_connections", ctx)))
+    )
+end
+
+
+function _M.inc_llm_active_connections(ctx)
+    inc_llm_active_connections(ctx, 1)
+end
+
+
+function _M.dec_llm_active_connections(ctx)
+    inc_llm_active_connections(ctx, -1)
+end
+
 function _M.get_prometheus()
     return prometheus
 end
