@@ -71,6 +71,8 @@ local _M = {
 
 
 function _M.check_schema(conf)
+    local check = {"endpoint_addr"}
+    core.utils.check_https(check, conf, plugin_name)
     return core.schema.check(schema, conf)
 end
 
